#include <glib-unix.h>
#include <assert.h>

#include "login-shell.h"
#include "wayland.h"

static gchar **g_shell_argv;

static void activate(GtkApplication *app, void *data)
{
    (void)app;
    (void)data;
}

static gboolean restart(gpointer d)
{
    (void)d;
    gint i, fdlimit;

    fdlimit = (int)sysconf(_SC_OPEN_MAX);
    g_debug("reload: closing fd's %d to %d", STDERR_FILENO + 1, fdlimit);
    for (i = STDERR_FILENO + 1; i < fdlimit; i++)
    {
        fcntl(i, F_SETFD, FD_CLOEXEC);
    }
    g_debug("reload: exec: %s", g_shell_argv[0]);
    execvp(g_shell_argv[0], g_shell_argv);
    exit(1);
    return FALSE;
}


static void startup(GtkApplication *app, DiyaLoginShell* shell)
{
    diya_shell_wayland_init(DIYA_SHELL(shell));
    g_unix_signal_add(SIGHUP, (GSourceFunc)restart, NULL);
    g_unix_signal_add(SIGINT,(GSourceFunc)g_application_quit,(void*)app);
    diya_login_shell_attach(shell, app);
}

int main(int argc, char *argv[])
{
    g_shell_argv = g_malloc0(sizeof(gchar *) * (argc + 1));
    for (int i = 0; i < argc; i++)
    {
        g_shell_argv[i] = argv[i];
    }
    GtkApplication* app = gtk_application_new("dev.iohub.diya.login-shell", G_APPLICATION_DEFAULT_FLAGS);
    
    DiyaLoginShell* shell = diya_login_shell_new();
    
    g_signal_connect(app, "startup", G_CALLBACK(startup), shell);
    g_signal_connect(app, "activate", G_CALLBACK(activate), shell);
    // g_signal_connect(app, "shutdown", G_CALLBACK(shutdown), (void*)shell);
    int status = g_application_run(G_APPLICATION(app), argc, argv);
    g_warning("Exiting SHELL");
    g_object_unref(shell);
    return status;
}
