#include "shell.h"
#include "wayland.h"
#include "virtual-keyboard.h"
enum
{
    NO_PROP,
    PROP_SHELL_WAYLAND,
    N_PROPERTIES
};

static GParamSpec *shell_properties[N_PROPERTIES] = {0};


typedef struct _DiyaShellPrivate
{
    DiyaObject  parent;
    DiyaWayland * wayland;
    DiyaVirtualKeyboard* vkb;
} DiyaShellPrivate;

G_DEFINE_TYPE_WITH_PRIVATE(DiyaShell, diya_shell, DIYA_TYPE_OBJECT);



static void diya_shell_dispose(GObject* object)
{
    g_debug("diya_shell_dispose: %s", diya_object_to_string(object));
    DiyaShell * self = DIYA_SHELL(object);
    DiyaShellPrivate* priv = diya_shell_get_instance_private(self);
    if(priv->wayland)
    {
        g_object_unref(priv->wayland);
    }
    G_OBJECT_CLASS(diya_shell_parent_class)->dispose(object);
}

static void diya_shell_set_property(GObject *object, guint property_id, const GValue *value, GParamSpec *pspec)
{
    DiyaShell * self = DIYA_SHELL(object);
    DiyaShellPrivate* priv = diya_shell_get_instance_private(self);

    switch (property_id)
    {
    case PROP_SHELL_WAYLAND:
        priv->wayland = g_value_get_pointer(value);
        break;
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
        break;
    }
}

static void diya_shell_get_property(GObject *object, guint property_id, GValue *value, GParamSpec *pspec)
{
    DiyaShell * self = DIYA_SHELL(object);
    DiyaShellPrivate* priv = diya_shell_get_instance_private(self);
    switch (property_id)
    {
    case PROP_SHELL_WAYLAND:
        g_value_set_pointer(value, priv->wayland);
        break;
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
        break;
    }
}


static void diya_shell_init(DiyaShell *self)
{
    DiyaShellPrivate* priv = diya_shell_get_instance_private(self);
    priv->wayland = NULL;
}

DiyaWayland * diya_shell_get_wayland(DiyaShell* shell)
{
    DiyaWayland * wayland;
    g_object_get(shell,"wayland", &wayland, NULL);
    assert(DIYA_IS_WAYLAND(wayland));
    return wayland;
}

static void diya_shell_class_init(DiyaShellClass *class)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS(class);
    gobject_class->dispose = diya_shell_dispose;
    gobject_class->set_property = diya_shell_set_property;
    gobject_class->get_property = diya_shell_get_property;

    class->foreign_register = NULL;
    class->virtual_keyboard_register = diya_virtual_keyboard_register;
 
    shell_properties[PROP_SHELL_WAYLAND] = g_param_spec_pointer("wayland", NULL, "Shell wayland", G_PARAM_READWRITE); // 
    g_object_class_install_properties (gobject_class, N_PROPERTIES, shell_properties);
}